@@ -1,7 +1,7 @@
 /*
  * entry.c
  *
- * Copyright (c) 2010-2011, 2014 The MacPorts Project
+ * Copyright (c) 2010-2011, 2014, 2017-2018 The MacPorts Project
  * Copyright (c) 2007 Chris Pickel <sfiera@macports.org>
  * All rights reserved.
  *
@@ -30,6 +30,11 @@
 #include <config.h>
 #endif
 
+/* required for asprintf(3) on macOS */
+#define _DARWIN_C_SOURCE
+/* required for asprintf(3) on Linux */
+#define _GNU_SOURCE
+
 #include "portgroup.h"
 #include "entry.h"
 #include "registry.h"
@@ -39,12 +44,9 @@
 #include <sqlite3.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 /*
- * TODO: possibly, allow reg_entry_search to take different matching strategies
- *       for different keys. I don't know of an application for this feature
- *       yet, so no reason to bother for now.
- *
  * TODO: reg_entry_installed and reg_entry_imaged could benefit from the added
  *       flexibility of -glob and -regexp too. Not a high priority, though.
  *
@@ -185,13 +187,13 @@ reg_entry* reg_entry_open(reg_registry* reg, char* name, char* version,
     char* query;
     if (strlen(epoch) > 0) {
         query = "SELECT id FROM registry.ports "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
                 "INDEXED BY port_name "
 #endif
                 "WHERE name=? AND version=? AND revision=? AND variants=? AND epoch=?";
     } else {
         query = "SELECT id FROM registry.ports "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
                 "INDEXED BY port_name "
 #endif
                 "WHERE name=? AND version=? AND revision=? AND variants=? AND epoch!=?";
@@ -249,6 +251,11 @@ reg_entry* reg_entry_open(reg_registry* reg, char* name, char* version,
 int reg_entry_delete(reg_entry* entry, reg_error* errPtr) {
     reg_registry* reg = entry->reg;
     int result = 0;
+
+    errPtr->code = REG_SQLITE_ERROR;
+    errPtr->description = "an unknown sqlite error occurred";
+    errPtr->free = NULL;
+
     sqlite3_stmt* ports = NULL;
     sqlite3_stmt* files = NULL;
     sqlite3_stmt* dependencies = NULL;
@@ -291,6 +298,7 @@ int reg_entry_delete(reg_entry* entry, reg_error* errPtr) {
                                                         case SQLITE_BUSY:
                                                             break;
                                                         case SQLITE_ERROR:
+                                                        default:
                                                             reg_sqlite_error(reg->db,
                                                                     errPtr, NULL);
                                                             break;
@@ -300,6 +308,7 @@ int reg_entry_delete(reg_entry* entry, reg_error* errPtr) {
                                             case SQLITE_BUSY:
                                                 break;
                                             case SQLITE_ERROR:
+                                            default:
                                                 reg_sqlite_error(reg->db,
                                                         errPtr, NULL);
                                                 break;
@@ -309,6 +318,7 @@ int reg_entry_delete(reg_entry* entry, reg_error* errPtr) {
                                 case SQLITE_BUSY:
                                     break;
                                 case SQLITE_ERROR:
+                                default:
                                     reg_sqlite_error(reg->db, errPtr, NULL);
                                     break;
                             }
@@ -323,6 +333,7 @@ int reg_entry_delete(reg_entry* entry, reg_error* errPtr) {
                 case SQLITE_BUSY:
                     break;
                 case SQLITE_ERROR:
+                default:
                     reg_sqlite_error(reg->db, errPtr, NULL);
                     break;
             }
@@ -390,23 +401,19 @@ static int reg_all_entries(reg_registry* reg, char* query, int query_len,
  * @param [in] keys      a list of keys to search by
  * @param [in] vals      a list of values to search by, matching keys
  * @param [in] key_count the number of key/value pairs passed
- * @param [in] strategy  strategy to use (one of the `reg_strategy_*` constants)
+ * @param [in] strategies  strategies to use (one of the `reg_strategy_*` constants)
  * @param [out] entries  a list of matching entries
  * @param [out] errPtr   on error, a description of the error that occurred
  * @return               the number of entries if success; false if failure
  */
-int reg_entry_search(reg_registry* reg, char** keys, char** vals, int key_count,
-        int strategy, reg_entry*** entries, reg_error* errPtr) {
+int reg_entry_search(reg_registry* reg, const char** keys, const char** vals,
+        int key_count, int *strategies, reg_entry*** entries, reg_error* errPtr) {
     int i;
     char* kwd = " WHERE ";
     char* query;
     size_t query_len, query_space;
     int result;
-    /* get the strategy */
-    char* op = reg_strategy_op(strategy, errPtr);
-    if (op == NULL) {
-        return -1;
-    }
+    char* op;
     /* build the query */
     query = strdup("SELECT id FROM registry.ports");
     if (!query) {
@@ -414,6 +421,11 @@ int reg_entry_search(reg_registry* reg, char** keys, char** vals, int key_count,
     }
     query_len = query_space = strlen(query);
     for (i=0; i<key_count; i++) {
+        /* get the strategy */
+        op = reg_strategy_op(strategies[i], errPtr);
+        if (op == NULL) {
+            return -1;
+        }
         char* cond = sqlite3_mprintf(op, keys[i], vals[i]);
         if (!cond || !reg_strcat(&query, &query_len, &query_space, kwd)
             || !reg_strcat(&query, &query_len, &query_space, cond)) {
@@ -500,7 +512,7 @@ int reg_entry_installed(reg_registry* reg, char* name, reg_entry*** entries,
         format = "%s WHERE state='installed'";
     } else {
         format = "%s "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
                 "INDEXED BY port_name "
 #endif
                 "WHERE state='installed' AND name='%q'";
@@ -517,16 +529,35 @@ int reg_entry_installed(reg_registry* reg, char* name, reg_entry*** entries,
  *
  * @param [in] reg     registry to search in
  * @param [in] path    path of the file to check ownership of
+ * @param [in] cs      false if check should be performed case-insensitive,
+ *                     true otherwise
  * @param [out] entry  the owner, or NULL if no active port owns the file
  * @param [out] errPtr on error, a description of the error that occurred
  * @return             true if success; false if failure
  */
-int reg_entry_owner(reg_registry* reg, char* path, reg_entry** entry,
+int reg_entry_owner(reg_registry* reg, char* path, int cs, reg_entry** entry,
         reg_error* errPtr) {
     int result = 0;
     sqlite3_stmt* stmt = NULL;
-    char* query = "SELECT id FROM registry.files WHERE actual_path=? AND active";
     int lower_bound = 0;
+    char *query = NULL;
+
+    asprintf(&query, "SELECT id FROM registry.files %s WHERE (actual_path = ? %s) AND active",
+#if MP_SQLITE_VERSION >= 3003013
+#if MP_SQLITE_VERSION >= 3006004
+             cs ? "INDEXED BY file_actual" : "INDEXED BY file_actual_nocase",
+#else
+             "",
+#endif
+             cs ? "" : "COLLATE NOCASE");
+#else
+             "", "");
+#endif
+
+    if (!query) {
+        return 0;
+    }
+
     if ((sqlite3_prepare_v2(reg->db, query, -1, &stmt, NULL) == SQLITE_OK)
             && (sqlite3_bind_text(stmt, 1, path, -1, SQLITE_STATIC)
                 == SQLITE_OK)) {
@@ -555,6 +586,7 @@ int reg_entry_owner(reg_registry* reg, char* path, reg_entry** entry,
     if (stmt) {
         sqlite3_finalize(stmt);
     }
+    free(query);
     return result;
 }
 
@@ -568,12 +600,31 @@ int reg_entry_owner(reg_registry* reg, char* path, reg_entry** entry,
  *
  * @param [in] reg  registry to find file in
  * @param [in] path path of file to get owner of
+ * @param [in] cs   false if check should be performed case-insensitive,
+ *                  true otherwise
  * @return          id of owner, or 0 for none
  */
-sqlite_int64 reg_entry_owner_id(reg_registry* reg, char* path) {
+sqlite_int64 reg_entry_owner_id(reg_registry* reg, char* path, int cs) {
     sqlite3_stmt* stmt = NULL;
     sqlite_int64 result = 0;
-    char* query = "SELECT id FROM registry.files WHERE actual_path=? AND active";
+    char *query = NULL;
+
+    asprintf(&query, "SELECT id FROM registry.files %s WHERE (actual_path = ? %s) AND active",
+#if MP_SQLITE_VERSION >= 3003013
+#if MP_SQLITE_VERSION >= 3006004
+             cs ? "INDEXED BY file_actual" : "INDEXED BY file_actual_nocase",
+#else
+             "",
+#endif
+             cs ? "" : "COLLATE NOCASE");
+#else
+             "", "");
+#endif
+
+    if (!query) {
+        return 0;
+    }
+
     if ((sqlite3_prepare_v2(reg->db, query, -1, &stmt, NULL) == SQLITE_OK)
             && (sqlite3_bind_text(stmt, 1, path, -1, SQLITE_STATIC)
                 == SQLITE_OK)) {
@@ -588,6 +639,7 @@ sqlite_int64 reg_entry_owner_id(reg_registry* reg, char* path) {
     if (stmt) {
         sqlite3_finalize(stmt);
     }
+    free(query);
     return result;
 }
 
@@ -821,7 +873,7 @@ int reg_entry_map(reg_entry* entry, char** files, int file_count,
 }
 
 /**
- * Unaps files from the given port in the filemap. The files must be owned by
+ * Unmaps files from the given port in the filemap. The files must be owned by
  * the given entry.
  *
  * @param [in] entry      the entry to unmap the files from
@@ -836,7 +888,7 @@ int reg_entry_unmap(reg_entry* entry, char** files, int file_count,
     int result = 1;
     sqlite3_stmt* stmt = NULL;
     char* query = "DELETE FROM registry.files "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
                   "INDEXED BY file_path "
 #endif
                   "WHERE path=? AND id=?";
@@ -852,8 +904,8 @@ int reg_entry_unmap(reg_entry* entry, char** files, int file_count,
                     switch (r) {
                         case SQLITE_DONE:
                             if (sqlite3_changes(reg->db) == 0) {
-                                reg_throw(errPtr, REG_INVALID, "this entry "
-                                        "does not own the given file");
+                                reg_throw(errPtr, REG_INVALID, "%s is not "
+                                        "owned by this entry", files[i]);
                                 result = 0;
                             } else {
                                 sqlite3_reset(stmt);
@@ -1041,7 +1093,7 @@ int reg_entry_activate(reg_entry* entry, char** files, char** as_files,
     char* select_query = "SELECT id FROM registry.files WHERE actual_path=? "
         "AND active";
     char* update_query = "UPDATE registry.files "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
         /* if the version of SQLite supports it force the usage of the index on
          * path, rather than the one on id which has a lot less discriminative
          * power and leads to very slow queries. This is needed for the new
@@ -1094,6 +1146,7 @@ int reg_entry_activate(reg_entry* entry, char** files, char** as_files,
                                         case SQLITE_BUSY:
                                             break;
                                         case SQLITE_ERROR:
+                                        default:
                                             reg_sqlite_error(reg->db, errPtr,
                                                     update_query);
                                             result = 0;
@@ -1104,6 +1157,7 @@ int reg_entry_activate(reg_entry* entry, char** files, char** as_files,
                             case SQLITE_BUSY:
                                 break;
                             case SQLITE_ERROR:
+                            default:
                                 reg_sqlite_error(reg->db, errPtr, select_query);
                                 result = 0;
                                 break;
@@ -1148,7 +1202,7 @@ int reg_entry_deactivate(reg_entry* entry, char** files, int file_count,
     int i;
     sqlite3_stmt* stmt = NULL;
     char* query = "UPDATE registry.files "
-#if SQLITE_VERSION_NUMBER >= 3006004
+#if MP_SQLITE_VERSION >= 3006004
         /* if the version of SQLite supports it force the usage of the index on
          * path, rather than the one on id which has a lot less discriminative
          * power and leads to very slow queries. This is needed for the new
@@ -1168,8 +1222,8 @@ int reg_entry_deactivate(reg_entry* entry, char** files, int file_count,
                     switch (r) {
                         case SQLITE_DONE:
                             if (sqlite3_changes(reg->db) == 0) {
-                                reg_throw(errPtr, REG_INVALID, "this entry "
-                                        "does not own the given file");
+                                reg_throw(errPtr, REG_INVALID, "%s is not "
+                                        "owned by this entry", files[i]);
                                 result = 0;
                             } else {
                                 sqlite3_reset(stmt);
