@@ -1,6 +1,5 @@
 /*
  * entryobj.c
- * $Id$
  *
  * Copyright (c) 2007 Chris Pickel <sfiera@macports.org>
  * All rights reserved.
@@ -40,15 +39,15 @@
 #include "util.h"
 
 const char* entry_props[] = {
+    "id",
     "name",
     "portfile",
-    "url",
     "location",
     "epoch",
     "version",
     "revision",
     "variants",
-    "negated_variants",
+    "requested_variants",
     "date",
     "state",
     "installtype",
@@ -56,6 +55,8 @@ const char* entry_props[] = {
     "os_platform",
     "os_major",
     "requested",
+    "cxx_stdlib",
+    "cxx_stdlib_overridden",
     NULL
 };
 
@@ -417,15 +418,15 @@ typedef struct {
 
 static entry_obj_cmd_type entry_cmds[] = {
     /* keys */
+    { "id", entry_obj_prop },
     { "name", entry_obj_prop },
     { "portfile", entry_obj_prop },
-    { "url", entry_obj_prop },
     { "location", entry_obj_prop },
     { "epoch", entry_obj_prop },
     { "version", entry_obj_prop },
     { "revision", entry_obj_prop },
     { "variants", entry_obj_prop },
-    { "negated_variants", entry_obj_prop },
+    { "requested_variants", entry_obj_prop },
     { "date", entry_obj_prop },
     { "state", entry_obj_prop },
     { "installtype", entry_obj_prop },
@@ -433,6 +434,8 @@ static entry_obj_cmd_type entry_cmds[] = {
     { "os_platform", entry_obj_prop },
     { "os_major", entry_obj_prop },
     { "requested", entry_obj_prop },
+    { "cxx_stdlib", entry_obj_prop },
+    { "cxx_stdlib_overridden", entry_obj_prop },
     /* filemap */
     { "map", entry_obj_filemap },
     { "unmap", entry_obj_filemap },
